//RadioProbe
//Copyright © 2000-2001, Paul Yu

#include "RadioProbeView.h"
#include "RadioProbeLooper.h"
#include "RadioProbeLink.h"
#include "RadioProbeProxy.h"
#include "RadioProbeConstants.h"

#include <Bitmap.h>
#include <Messenger.h>
#include <MessageRunner.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Menu.h>
#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <Dragger.h>
#include <SupportDefs.h>
#include <time.h>
#include <stdio.h>
#include <MenuField.h>
#include kIconFile

RadioProbeView::RadioProbeView(BRect frame)
	: BView(frame, kReplicantName, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS)
{
	SetViewColor(kViewColor);
	SetLowColor(kViewColor);
	SetHighColor(kTextColor);
	mPaused = false;

	Init();
}

RadioProbeView::RadioProbeView(BMessage *archive) : BView(archive)
{
	BMessage views;
	const void *linkcolors;
	const void *wscolors;
	ssize_t size;
	int32 stream;
	
	archive->FindMessage("RPProxyInfo", &mProxyInfo);

	if (archive->FindBool("Paused", &mPaused) < B_OK)
		mPaused = false;

	Init();

	if (archive->FindData("LinkColors", 'RGBC', &linkcolors, &size) == B_OK)
		memcpy(mLink->mLinkColors, linkcolors, sizeof(mLink->mLinkColors));

	if (archive->FindData("WSColors", 'RGBC', &wscolors, &size) == B_OK)
		memcpy(mWSColors, wscolors, sizeof(mWSColors));
		
	if (archive->FindInt32("Stream", &stream) == B_OK)
	{
		BMenuItem *item = mStream->ItemAt(stream);
		if (item)
			item->SetMarked(true);
	}
}

void RadioProbeView::SetColors()
{
	Invalidate();
	mMenufield->Invalidate();
	mLink->Invalidate();

	int32 current_ws = current_workspace();
	rgb_color current = mWSColors[current_ws][1];

	SetViewColor(current);
	SetLowColor(current);
	SetHighColor(mWSColors[current_ws][0]);

	mLink->SetViewColor(current);
	mLink->SetLowColor(current);
	mLink->SetHighColor(mWSColors[current_ws][0]);

	mMenufield->SetViewColor(current);
	mMenufield->SetLowColor(current);
}
	
void RadioProbeView::Init()
{
	app_info ai;
	BDragger *dragger;
	BRect frame;

	SetFont(be_bold_font);
	SetFontSize(12);
	mPrevWorkspace = -1;

	mBitmap = new BBitmap(BRect(0, 0, kImageWidth-1, kImageHeight-1), kImageColorSpace);
	mBitmap->SetBits((const void *)kImageBits, sizeof(kImageBits), 0, kImageColorSpace);
	
	if (mPaused)
	{
		mTitle.SetTo(B_EMPTY_STRING);
		mArtist.SetTo(B_EMPTY_STRING);
	}
	else
	{
		mTitle.SetTo(CONNEXION);
		mArtist.SetTo(NETWORK_SERVER);
	}
	
	mMessage.what = P_UPDATE_CURRENT_SONG;
	mMessage.AddPointer("view", (void *)this);

	mLooper = new RadioProbeLooper();

	mLink = new RadioProbeLink(mLooper);
	AddChild(mLink);

	if (be_app->GetAppInfo(&ai) < B_OK)
		throw B_ERROR;
		
	if (strcmp(ai.signature, kRadioProbeAppSig) != 0)
	{
		frame = Bounds();
		SetResizingMode(B_FOLLOW_NONE);
		dragger = new BDragger(BRect(frame.right-7,frame.bottom-7,frame.right,frame.bottom), this,  B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW);
		AddChild(dragger);
	}

	frame = mLink->Frame();
	BMenuItem *item;
	mStream = new BPopUpMenu("", true, true, B_ITEMS_IN_COLUMN);
	mMenufield = new BMenuField(BRect(frame.right + 5, frame.top-5, frame.right + 95, frame.bottom), "menu", NULL, mStream);
	mMenufield->SetDivider(0.0);
	AddChild(mMenufield);
	item = new BMenuItem("24K mono", new BMessage(P_SET_STREAM));
	mStream->AddItem(item);
	item->SetMarked(true);
	item = new BMenuItem("32K mono", new BMessage(P_SET_STREAM));
	mStream->AddItem(item);
	item = new BMenuItem("56K stereo", new BMessage(P_SET_STREAM));
	mStream->AddItem(item);

	mLooper->Run();

	mProxyInfo.what = P_SET_SERVER;
	mLooper->PostMessage(&mProxyInfo);

	rgb_color text_color = { kTextColor };
	rgb_color view_color = { kViewColor };

	for(int i = 0; i < 32; i++)
		memcpy(&mWSColors[i][0], &text_color, sizeof(rgb_color));
	for(int j = 0; j < 32; j++)
		memcpy(&mWSColors[j][1], &view_color, sizeof(rgb_color));

	if (mPaused)
		mRunner = NULL;
	else
		mRunner = new BMessageRunner(BMessenger(mLooper), &mMessage, kRefreshRate);
}

RadioProbeView::~RadioProbeView()
{
	delete mRunner;

	//Correct but Tracker doesn't know how to kill replicants asynchronously
	//if (mLooper->Lock())
	//	mLooper->Quit();
	
	//the work-around will crash containers that unload the application image immediately after deleting it
	mLooper->PostMessage(B_QUIT_REQUESTED);

	delete mBitmap;
}

void RadioProbeView::AttachedToWindow()
{
	mStream->SetTargetForItems(this);

	if (!mPaused)
		mLooper->PostMessage(&mMessage);

	Looper()->PostMessage(P_SET_STREAM, this);
}

void RadioProbeView::FrameResized(float width, float height)
{
	BRect bounds = Window()->Bounds();

	ResizeTo(bounds.right, bounds.bottom);
	Invalidate();
}

void RadioProbeView::Draw(BRect updateRect)
{
	int32 current_ws = current_workspace();

	if (mPrevWorkspace != current_ws)
	{
		mPrevWorkspace = current_ws;
		Looper()->PostMessage(P_SET_COLORS, this);
		return;
	}

	rgb_color current = mWSColors[current_ws][1];
	rgb_color original = { kViewColor, 0 };
	if (current.red == original.red && current.green == original.green && current.blue == original.blue)
	{
		int32 vpos;
		BRect frame = Frame();
		BView *menu = FindView("menu");
		BRect mframe = menu->Frame();
		
		float bottom = min_c(frame.bottom, Window()->Bounds().bottom) + 1;
		if ((frame.right - frame.left) > mframe.right + kImageWidth)
			vpos = (int32)(bottom - frame.top - kImageHeight) / 2;
		else
			vpos = 1;

		DrawBitmap(mBitmap, BPoint(Bounds().right-kImageWidth+1, vpos));
	}

	BFont font(be_bold_font);
	font.SetSize(12);
	float label_width = 11 + max_c(font.StringWidth(TITLE), font.StringWidth(ARTIST));

	DrawString(TITLE, BPoint(7,20));
	DrawString(mTitle.String(), BPoint(label_width,20));
	DrawString(ARTIST, BPoint(7,37));
	DrawString(mArtist.String(), BPoint(label_width,37));
}

void RadioProbeView::MouseDown(BPoint where)
{
	BPoint location;
	uint32 buttons;
	BWindow *window;
	BRect frame;
	app_info ai;
	bool replicant;

	if ((be_app->GetAppInfo(&ai) == B_OK) &&
		(strcmp(ai.signature, kRadioProbeAppSig) == 0))
		replicant = false;
	else
		replicant = true;

	window = Window();
	GetMouse(&location, &buttons);

	if ((window != NULL) && (buttons & B_SECONDARY_MOUSE_BUTTON))
	{
		BPopUpMenu *menu = new BPopUpMenu(PROPERTIES, false, false, B_ITEMS_IN_COLUMN);
		BMenuItem *item;

		item = new BMenuItem(ABOUT, new BMessage(B_ABOUT_REQUESTED));
		item->SetTarget(this);
		menu->AddItem(item);

		menu->AddSeparatorItem();

		if (!replicant)
		{
			item = new BMenuItem(ALWAYS_ON_TOP, new BMessage(P_ALWAYS_ON_TOP));
			item->SetTarget(this);
			menu->AddItem(item);
			item->SetMarked(window->Feel() == B_FLOATING_ALL_WINDOW_FEEL);
	
			item = new BMenuItem(SHOW_ON_ALL_WORKSPACES, new BMessage(P_ALL_WORKSPACES));
			item->SetTarget(this);
			menu->AddItem(item);
			item->SetMarked(window->Workspaces() == B_ALL_WORKSPACES);
	
			BMenu *submenu = new BMenu(TITLE_AND_BORDERS, B_ITEMS_IN_COLUMN);
			item->SetTarget(this);
			menu->AddItem(submenu);
	
			window_look look = window->Look();
			item = new BMenuItem(NORMAL, new BMessage(P_NORMAL_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_TITLED_WINDOW_LOOK);
	
			item = new BMenuItem(LITTLE, new BMessage(P_SMALL_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_FLOATING_WINDOW_LOOK);
	
			item = new BMenuItem(THICK_BORDER, new BMessage(P_THICK_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_MODAL_WINDOW_LOOK);
	
			item = new BMenuItem(OUTLINE_BORDER, new BMessage(P_OUTLINE_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_BORDERED_WINDOW_LOOK);
	
			item = new BMenuItem(NO_BORDER, new BMessage(P_NO_BORDER));
			item->SetTarget(this);
			submenu->AddItem(item);
			item->SetMarked(look == B_NO_BORDER_WINDOW_LOOK);
		}

		item = new BMenuItem(PROXY, new BMessage(P_PROXY_SETUP));
		item->SetTarget(this);
		menu->AddItem(item);

		item = new BMenuItem(PAUSE_UPDATING, new BMessage(P_PAUSE_UPDATE));
		item->SetTarget(this);
		menu->AddItem(item);
		item->SetMarked(mPaused);

		if (!replicant)
		{
			menu->AddSeparatorItem();
	
			item = new BMenuItem(EXIT, new BMessage(B_QUIT_REQUESTED));
			item->SetTarget(this);
			menu->AddItem(item);
		}

		menu->Go(ConvertToScreen(where), true, true, ConvertToScreen(BRect(where.x-1, where.y-1, where.x+1, where.y+1)), false);
		delete menu;
	}
	else if ((!replicant) & (buttons & B_PRIMARY_MOUSE_BUTTON))
	{
		BMessage drag_message('ARCV');
		
		frame = Bounds();
		frame.OffsetBy(location.x - frame.right, location.y - frame.bottom);
		if (Archive(&drag_message, false) == B_OK)
			DragMessage(&drag_message, frame, NULL);
	}
	else
		BView::MouseDown(where);
}

void RadioProbeView::MessageReceived(BMessage *msg)
{
	BWindow *window = Window();
	const char *title, *artist;

	switch(msg->what)
	{
		case P_SET_COLORS:
		{
			SetColors();
		}
		break;
	
		case P_SET_STREAM:
		{
			BMenuItem *item = mStream->FindMarked();
			BMessage change(P_SET_STREAM);

			switch(mStream->IndexOf(item))
			{
				case 0:
					change.AddString("stream", kLowURL); break;
				
				case 1:
					change.AddString("stream", kMedURL); break;

				case 2:
					change.AddString("stream", kHighURL); break;
			}
			
			mLooper->PostMessage(&change);
		}
		break;
	
		case B_PASTE:
		{
			rgb_color *rgb;
			ssize_t size;
			BPoint point;

			if (msg->FindPoint("_drop_point_", &point) == B_OK)
			{
				BFont font(be_bold_font);
				float title_length = font.StringWidth(mTitle.String());
				float artist_length = font.StringWidth(mArtist.String());
			
				BRect text_area(7, 10, 55 + max_c(title_length, artist_length), 49);
				point = ConvertFromScreen(point);

				if (msg->FindData("RGBColor", 'RGBC', (const void **)&rgb, &size) == B_OK)
				{
					if (text_area.Contains(point))
						mWSColors[current_workspace()][0] = *rgb;
					else
						mWSColors[current_workspace()][1] = *rgb;
					
					SetColors();
				}
			}
		}
		break;
	
		case P_UPDATE_CURRENT_SONG:
		{
			if ((msg->FindString("title", &title) == B_OK) &&
				(msg->FindString("artist", &artist) == B_OK))
			{
				mTitle.SetTo(title);
				mArtist.SetTo(artist);
				Invalidate();
			}
		}
		break;

		case P_ALWAYS_ON_TOP:
		{
			if (window->Feel() == B_FLOATING_ALL_WINDOW_FEEL)
				window->SetFeel(B_NORMAL_WINDOW_FEEL);
			else
				window->SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
		}
		break;

		case P_ALL_WORKSPACES:
		{
			if (window->Workspaces() == B_ALL_WORKSPACES)
				window->SetWorkspaces(B_CURRENT_WORKSPACE);
			else
				window->SetWorkspaces(B_ALL_WORKSPACES);
		}
		break;

		case P_NORMAL_BORDER:
			window->SetLook(B_TITLED_WINDOW_LOOK); break;

		case P_SMALL_BORDER:
			window->SetLook(B_FLOATING_WINDOW_LOOK); break;

		case P_THICK_BORDER:
			window->SetLook(B_MODAL_WINDOW_LOOK); break;

		case P_OUTLINE_BORDER:
			window->SetLook(B_BORDERED_WINDOW_LOOK); break;
			
		case P_NO_BORDER:
			window->SetLook(B_NO_BORDER_WINDOW_LOOK); break;

		case B_QUIT_REQUESTED:
		{
			app_info ai;
			BMessage note;
		
			if (be_app->GetAppInfo(&ai) == B_OK)
			{
				if (strcmp(ai.signature, kRadioProbeAppSig) == 0)
					be_app->PostMessage(B_QUIT_REQUESTED);
			}
		}
		break;

		case B_ABOUT_REQUESTED:
		{
			BString string;

			string.SetTo(ABOUT_LINE_1 "\n" ABOUT_LINE_2 "\n\n" ABOUT_LINE_3 "\n");

//#ifdef BeOSRADIO
//			char times[6];
//			string.Append("\n" ABOUT_LINE_4 " ");
//			tzset();
//			sprintf(times, "%02ld:%02ld", 
//				(((18 * 3600) - timezone) / 3600) % 24,		//18:00 GMT
//				((24 * 3600) - timezone / 60) % 60);
//			string.Append(times);
//			string.Append(" " TO " ");
//			sprintf(times, "%02ld:%02ld",
//				(((6 * 3600) - timezone) / 3600) % 24,		//06:00 GMT
//				((24 * 3600) - timezone / 60) % 60);
//			string.Append(times);
//			string.Append(" " ABOUT_LINE_5 "\n");
//
//#endif

			(new BAlert(ABOUT, string.String(), FREEWARE))->Go();
		}
		break;

		case P_PROXY_SETUP:
		{
			RadioProbeProxy *proxy_window = new RadioProbeProxy(this, &mProxyInfo);
			proxy_window->Show();
		}
		break;

		case P_PAUSE_UPDATE:
		{
			if (mPaused)
			{
				mPaused = false;
				mRunner = new BMessageRunner(BMessenger(mLooper), &mMessage, kRefreshRate);
				mLooper->PostMessage(&mMessage);
				mTitle.SetTo(CONNEXION);
				mArtist.SetTo(NETWORK_SERVER);
			}
			else
			{
				mPaused = true;
				delete mRunner;
				mRunner = NULL;
				mTitle.SetTo(B_EMPTY_STRING);
				mArtist.SetTo(B_EMPTY_STRING);
			}
			Invalidate();
		}
		break;

		case P_SET_SERVER:
		{
			try
			{
				mProxyInfo = *msg;
				mLooper->PostMessage(msg);
			}
			catch(...)
			{
				/*	*/
			}
		}
		break;
				
		default:
			BView::MessageReceived(msg);
	}
}

status_t RadioProbeView::Archive(BMessage *into, bool deep) const
{
	status_t err;
	BMenuItem *item = mStream->FindMarked();
	int32 stream = mStream->IndexOf(item);

	if (((err = BView::Archive(into, false)) == B_OK) &&
		((err = into->AddMessage("RPProxyInfo", &mProxyInfo)) == B_OK) &&
		((err = into->AddData("WSColors", 'RGBC', mWSColors, sizeof(mWSColors))) == B_OK) &&
		((err = into->AddData("LinkColors", 'RGBC', &mLink->mLinkColors, sizeof(mLink->mLinkColors))) == B_OK) &&
		((err = into->AddBool("Paused", mPaused)) == B_OK) &&
		((err = into->AddInt32("Stream", stream)) == B_OK))
		err = into->AddString("add_on", kRadioProbeAppSig);
	return err;
}

BArchivable	*RadioProbeView::Instantiate(BMessage *from)
{
	if (validate_instantiation(from, "RadioProbeView"))
		return new RadioProbeView(from);
	return NULL;
}
